/* Generated by vala-dbus-binding-tool 1.0-aa2fb. Do not modify! */
/* Generated with: vala-dbus-binding-tool --api-path=./org.gnome.Shell.Screencast.xml --directory=./ --strip-namespace=org --rename-namespace=gnome:Gnome --rename-namespace=Shell:ShellNS --no-synced */
using GLib;

namespace Gnome {

	namespace ShellNS {

		[DBus (name = "org.gnome.Shell.Screencast", timeout = 120000)]
		public interface Screencast : GLib.Object {

			[DBus (name = "Screencast")]
			public abstract void screencast(string file_template, GLib.HashTable<string, GLib.Variant> options, out bool success, out string filename_used) throws DBusError, IOError;

			[DBus (name = "ScreencastArea")]
			public abstract void screencast_area(int x, int y, int width, int height, string file_template, GLib.HashTable<string, GLib.Variant> options, out bool success, out string filename_used) throws DBusError, IOError;

			[DBus (name = "StopScreencast")]
			public abstract bool stop_screencast() throws DBusError, IOError;
		}
	}
}
