/* Generated by vala-dbus-binding-tool 1.0-aa2fb. Do not modify! */
/* Generated with: vala-dbus-binding-tool --api-path=./org.freedesktop.FileManager1.xml --directory=./ --strip-namespace=org --rename-namespace=freedesktop:Freedesktop --no-synced */
using GLib;

namespace Freedesktop {

	[DBus (name = "org.freedesktop.FileManager1", timeout = 120000)]
	public interface FileManager1 : GLib.Object {

		[DBus (name = "ShowFolders")]
		public abstract void show_folders(string[] URIs, string StartupId) throws DBusError, IOError;

		[DBus (name = "ShowItems")]
		public abstract void show_items(string[] URIs, string StartupId) throws DBusError, IOError;

		[DBus (name = "ShowItemProperties")]
		public abstract void show_item_properties(string[] URIs, string StartupId) throws DBusError, IOError;
	}
}
