/*
Peek Copyright (c) 2017 by Philipp Wolfer <ph.wolfer@gmail.com>

This file is part of Peek.

This software is licensed under the GNU General Public License
(version 3 or later). See the LICENSE file in this distribution.
*/

namespace Peek {

  public errordomain PeekError {
    UNKNOWN_SCREEN_RECORDER,
    NO_SUITABLE_SCREEN_RECORDER,
    SCREEN_RECORDER_ERROR
  }

  public errordomain RecordingError {
    INITIALIZING_RECORDING_FAILED,
    RECORDING_ABORTED,
    POSTPROCESSING_ABORTED
  }

}
